% Author: guibv.av...@gmail.com https://code.google.com/p/tictactoe-prolog/
% Modified: Diego & Yamandú 

:- module(heuristics,
        [ value/5,               % The heuristic for board state
          win/3,                  % Whether some player wins
          fichaEnPuntoMedio/3
	    ]).

:- use_module(molinero,
        [ rivalSinFichas/1,
          rivalBloqueado/2
        ]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%%     Heuristics
%%%
%%%
%%%     The h_func calculates the heuristics function for a
%%%     given position of the board.
%%%
%%%         h_func(+Board, +Player, -Weight)
%%%
%%%         where Board is the current board state
%%%               Player is the piece of current player (x / / o )
%%%               Weight is the heuristic value


% Antigua
% h_func(Board, Player, G) :-
%	win(Board,Player,G).

% win(Board, Player, G) :-
%	((molinero:rivalSinFichas(Board, Player) | molinero:rivalBloqueado(Board, Player))->
%		G is 50
%	;
%		G is 0
%	).
	
	%%%%%%% NUEVA %%
	
h_func(Board, Player, W, ListaFichas, ListaFichasRival) :-
	win(Board,Player,S1),
	fichaPorNivel(Board,Player,S2),
	fichaEnPuntoMedio(Board,Player,S3),
	masFichas(ListaFichas, ListaFichasRival, S4), 
	molinoFormado(Board,Player,ListaFichas,S5),
	W is S1+S2+S3+S4+S5.

win(Board, Player, S) :-
	( (molinero:rivalSinFichas( Player) | molinero:rivalBloqueado(Board, Player) )->
		S = 900000
	;
		S = 0
	).

fichaPorNivel(Board, Player, S) :-
	( molinero:unaFichaEnCadaNivel(Board,Player) ->
		S = 200
	;
		S = 0
	).

fichaEnPuntoMedio(Board, Player, S) :-
	molinero:fichasEnPuntosMedios(Board,Player,N),
	S is N*30.

	
molinoFormado(Board, Player, ListaFichas, S) :-	
	findall(X,(member(X,ListaFichas),molinero:molinoFormado(Board,X,Player)),Lista),
	molinero:longitud(Lista,Long),
	S = Long*80000.
	
masFichas(ListaFichas, ListaFichasRival, S):-	
	molinero:longitud(ListaFichasRival,LongR),
	molinero:longitud(ListaFichas,Long),
	S = (9 - LongR)*10000 + Long*10000.
	%%%%%


value(Board, Player, Val, X1, X2) :-
  h_func(Board, 1, W1, X1, X2),
  h_func(Board, 2, W2, X2, X1),
  scoreof(W1, W2, Player, Val).

scoreof(W1, W2, 1, Val) :-
  Val is W1-W2, !.

scoreof(W1, W2, _, Val) :-
  Val is W2-W1.

